# Generated by roxygen2: do not edit by hand

S3method(coef,APCS)
S3method(coef,CAES)
S3method(coef,CBDCS)
S3method(coef,CBDQCS)
S3method(coef,CBDS)
S3method(coef,CFM2S)
S3method(coef,CFMS)
S3method(coef,Fit1)
S3method(coef,Fit10)
S3method(coef,Fit11)
S3method(coef,Fit12)
S3method(coef,Fit13)
S3method(coef,Fit14)
S3method(coef,Fit2)
S3method(coef,Fit3)
S3method(coef,Fit4)
S3method(coef,Fit5)
S3method(coef,Fit6)
S3method(coef,Fit7)
S3method(coef,Fit8)
S3method(coef,Fit9)
S3method(coef,LCLS)
S3method(coef,LCS)
S3method(coef,PAPCS)
S3method(coef,PCAES)
S3method(coef,PCBDCS)
S3method(coef,PCBDQCS)
S3method(coef,PCBDS)
S3method(coef,PCFM2S)
S3method(coef,PCFMS)
S3method(coef,PLCS)
S3method(coef,PRHS)
S3method(coef,RHS)
S3method(coef,STARS)
S3method(coef,cFit1)
S3method(coef,cFit10)
S3method(coef,cFit11)
S3method(coef,cFit12)
S3method(coef,cFit13)
S3method(coef,cFit14)
S3method(coef,cFit2)
S3method(coef,cFit3)
S3method(coef,cFit4)
S3method(coef,cFit5)
S3method(coef,cFit6)
S3method(coef,cFit7)
S3method(coef,cFit8)
S3method(coef,cFit9)
S3method(deviance,Fit1)
S3method(deviance,Fit10)
S3method(deviance,Fit11)
S3method(deviance,Fit12)
S3method(deviance,Fit13)
S3method(deviance,Fit14)
S3method(deviance,Fit2)
S3method(deviance,Fit3)
S3method(deviance,Fit4)
S3method(deviance,Fit5)
S3method(deviance,Fit6)
S3method(deviance,Fit7)
S3method(deviance,Fit8)
S3method(deviance,Fit9)
S3method(deviance,cFit1)
S3method(deviance,cFit10)
S3method(deviance,cFit11)
S3method(deviance,cFit12)
S3method(deviance,cFit13)
S3method(deviance,cFit14)
S3method(deviance,cFit2)
S3method(deviance,cFit3)
S3method(deviance,cFit4)
S3method(deviance,cFit5)
S3method(deviance,cFit6)
S3method(deviance,cFit7)
S3method(deviance,cFit8)
S3method(deviance,cFit9)
S3method(fitted,Fit1)
S3method(fitted,Fit10)
S3method(fitted,Fit11)
S3method(fitted,Fit12)
S3method(fitted,Fit13)
S3method(fitted,Fit14)
S3method(fitted,Fit2)
S3method(fitted,Fit3)
S3method(fitted,Fit4)
S3method(fitted,Fit5)
S3method(fitted,Fit6)
S3method(fitted,Fit7)
S3method(fitted,Fit8)
S3method(fitted,Fit9)
S3method(fitted,cFit1)
S3method(fitted,cFit10)
S3method(fitted,cFit11)
S3method(fitted,cFit12)
S3method(fitted,cFit13)
S3method(fitted,cFit14)
S3method(fitted,cFit2)
S3method(fitted,cFit3)
S3method(fitted,cFit4)
S3method(fitted,cFit5)
S3method(fitted,cFit6)
S3method(fitted,cFit7)
S3method(fitted,cFit8)
S3method(fitted,cFit9)
S3method(forecast,APCS)
S3method(forecast,CAES)
S3method(forecast,CBDCS)
S3method(forecast,CBDQCS)
S3method(forecast,CBDS)
S3method(forecast,CFM2S)
S3method(forecast,CFMS)
S3method(forecast,ENI)
S3method(forecast,ENS)
S3method(forecast,LCLS)
S3method(forecast,LCS)
S3method(forecast,PAPCS)
S3method(forecast,PCAES)
S3method(forecast,PCBDCS)
S3method(forecast,PCBDQCS)
S3method(forecast,PCBDS)
S3method(forecast,PCFM2S)
S3method(forecast,PCFMS)
S3method(forecast,PLCS)
S3method(forecast,PRHS)
S3method(forecast,RHS)
S3method(forecast,STARS)
S3method(plot,APCS)
S3method(plot,CAES)
S3method(plot,CBDCS)
S3method(plot,CBDQCS)
S3method(plot,CBDS)
S3method(plot,CFM2S)
S3method(plot,CFMS)
S3method(plot,ENI)
S3method(plot,ENS)
S3method(plot,Fit1)
S3method(plot,Fit10)
S3method(plot,Fit11)
S3method(plot,Fit12)
S3method(plot,Fit13)
S3method(plot,Fit14)
S3method(plot,Fit2)
S3method(plot,Fit3)
S3method(plot,Fit4)
S3method(plot,Fit5)
S3method(plot,Fit6)
S3method(plot,Fit7)
S3method(plot,Fit8)
S3method(plot,Fit9)
S3method(plot,LCLS)
S3method(plot,LCS)
S3method(plot,PAPCS)
S3method(plot,PCAES)
S3method(plot,PCBDCS)
S3method(plot,PCBDQCS)
S3method(plot,PCBDS)
S3method(plot,PCFM2S)
S3method(plot,PCFMS)
S3method(plot,PLCS)
S3method(plot,PRHS)
S3method(plot,RHS)
S3method(plot,STARS)
S3method(plot,cFit1)
S3method(plot,cFit10)
S3method(plot,cFit11)
S3method(plot,cFit12)
S3method(plot,cFit13)
S3method(plot,cFit14)
S3method(plot,cFit2)
S3method(plot,cFit3)
S3method(plot,cFit4)
S3method(plot,cFit5)
S3method(plot,cFit6)
S3method(plot,cFit7)
S3method(plot,cFit8)
S3method(plot,cFit9)
S3method(predict,Fit1)
S3method(predict,Fit10)
S3method(predict,Fit11)
S3method(predict,Fit12)
S3method(predict,Fit13)
S3method(predict,Fit14)
S3method(predict,Fit2)
S3method(predict,Fit3)
S3method(predict,Fit4)
S3method(predict,Fit5)
S3method(predict,Fit6)
S3method(predict,Fit7)
S3method(predict,Fit8)
S3method(predict,Fit9)
S3method(predict,cFit1)
S3method(predict,cFit10)
S3method(predict,cFit11)
S3method(predict,cFit12)
S3method(predict,cFit13)
S3method(predict,cFit14)
S3method(predict,cFit2)
S3method(predict,cFit3)
S3method(predict,cFit4)
S3method(predict,cFit5)
S3method(predict,cFit6)
S3method(predict,cFit7)
S3method(predict,cFit8)
S3method(predict,cFit9)
S3method(residuals,APCS)
S3method(residuals,CAES)
S3method(residuals,CBDCS)
S3method(residuals,CBDQCS)
S3method(residuals,CBDS)
S3method(residuals,CFM2S)
S3method(residuals,CFMS)
S3method(residuals,Fit1)
S3method(residuals,Fit10)
S3method(residuals,Fit11)
S3method(residuals,Fit12)
S3method(residuals,Fit13)
S3method(residuals,Fit14)
S3method(residuals,Fit2)
S3method(residuals,Fit3)
S3method(residuals,Fit4)
S3method(residuals,Fit5)
S3method(residuals,Fit6)
S3method(residuals,Fit7)
S3method(residuals,Fit8)
S3method(residuals,Fit9)
S3method(residuals,LCLS)
S3method(residuals,LCS)
S3method(residuals,PAPCS)
S3method(residuals,PCAES)
S3method(residuals,PCBDCS)
S3method(residuals,PCBDQCS)
S3method(residuals,PCBDS)
S3method(residuals,PCFM2S)
S3method(residuals,PCFMS)
S3method(residuals,PLCS)
S3method(residuals,PRHS)
S3method(residuals,RHS)
S3method(residuals,STARS)
S3method(residuals,cFit1)
S3method(residuals,cFit10)
S3method(residuals,cFit11)
S3method(residuals,cFit12)
S3method(residuals,cFit13)
S3method(residuals,cFit14)
S3method(residuals,cFit2)
S3method(residuals,cFit3)
S3method(residuals,cFit4)
S3method(residuals,cFit5)
S3method(residuals,cFit6)
S3method(residuals,cFit7)
S3method(residuals,cFit8)
S3method(residuals,cFit9)
S3method(simulate,APCS)
S3method(simulate,CBDCS)
S3method(simulate,CBDQCS)
S3method(simulate,CBDS)
S3method(simulate,LCLS)
S3method(simulate,LCS)
S3method(simulate,RHS)
S3method(simulate,STARS)
export(APCS)
export(CAES)
export(CBDCS)
export(CBDQCS)
export(CBDS)
export(CC)
export(CFM2S)
export(CFMS)
export(ENI)
export(ENS)
export(FCS)
export(LCLS)
export(LCS)
export(MC)
export(PAPCS)
export(PCAES)
export(PCBDCS)
export(PCBDQCS)
export(PCBDS)
export(PCFM2S)
export(PCFMS)
export(PFCS)
export(PLCS)
export(PRHS)
export(RHS)
export(STARS)
export(mortalitymatrix)
importFrom(MASS,mvrnorm)
importFrom(MortalityLaws,MortalityLaw)
importFrom(NlcOptim,solnl)
importFrom(forecast,auto.arima)
importFrom(forecast,forecast)
importFrom(forecast,tsclean)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(minpack.lm,nls.lm)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,fitted)
importFrom(stats,lm)
importFrom(stats,nlminb)
importFrom(stats,optim)
importFrom(stats,prcomp)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,simulate)
